package model

import (
	"github.com/astaxie/beego/orm"
	_ "github.com/go-sql-driver/mysql"
	"time"
)

type UserLevelTaskType struct {
	ID                  int64     `gorm:"column:id" db:"id" json:"id" form:"id"`
	FreeCallCount       int64     `gorm:"column:free_call_count" db:"free_call_count" json:"free_call_count" form:"free_call_count"`
	UserLevelId         int64     `gorm:"column:user_level_id" db:"user_level_id" json:"user_level_id" form:"user_level_id"`
	TaskTypeId          int64     `gorm:"column:task_type_id" db:"task_type_id" json:"task_type_id" form:"task_type_id"`
	FreeCallCountPeriod int64     `gorm:"column:free_call_count_period" db:"free_call_count_period" json:"free_call_count_period" form:"free_call_count_period"`
	CreatedTime         time.Time `gorm:"column:created_time" db:"created_time" json:"created_time" form:"created_time"`
	UpdatedTime         time.Time `gorm:"column:updated_time" db:"updated_time" json:"updated_time" form:"updated_time"`
	Deleted             int64     `gorm:"column:deleted" db:"deleted" json:"deleted" form:"deleted"`
}

func (UserLevelTaskType) TableName() string {
	return "user_level_task_type"
}

type UserLevelTaskTypeRepository interface {
	Create(user *UserLevelTaskType) error
	GetById(uid int64) (*UserLevelTaskType, error)
	GetListByFilter(filters ...interface{}) []*UserLevelTaskType
	GetByTaskTypeAndUserLevelId(taskType int64, userLevel int64) (*UserLevelTaskType, error)
	Update(user *UserLevelTaskType) error
	Delete(user *UserLevelTaskType) error
}

type userLevelTaskTypeRepositoryImpl struct {
	o orm.Ormer
}

func NewUserLevelTaskTypeRepository() UserLevelTaskTypeRepository {
	o := orm.NewOrm()
	return &userLevelTaskTypeRepositoryImpl{o}
}

func (repo *userLevelTaskTypeRepositoryImpl) Create(user *UserLevelTaskType) error {
	_, err := repo.o.Insert(user)
	return err
}

func (repo *userLevelTaskTypeRepositoryImpl) GetListByFilter(filters ...interface{}) []*UserLevelTaskType {
	list := make([]*UserLevelTaskType, 0)
	query := orm.NewOrm().QueryTable(new(UserLevelTaskType).TableName())
	if len(filters) > 0 {
		l := len(filters)
		for k := 0; k < l; k += 2 {
			query = query.Filter(filters[k].(string), filters[k+1])
		}
	}
	query.OrderBy("-id").All(&list)
	//total, _ := query.Count()
	return list
}

func (repo *userLevelTaskTypeRepositoryImpl) GetByTaskTypeAndUserLevelId(taskType int64, userLevelId int64) (*UserLevelTaskType, error) {
	user := UserLevelTaskType{TaskTypeId: taskType, UserLevelId: userLevelId}
	err := repo.o.Read(&user)
	if err != nil {
		return nil, err
	}
	return &user, nil
}

func (repo *userLevelTaskTypeRepositoryImpl) GetById(uid int64) (*UserLevelTaskType, error) {
	user := UserLevelTaskType{ID: uid}
	err := repo.o.Read(&user)
	if err != nil {
		return nil, err
	}
	return &user, nil
}

func (repo *userLevelTaskTypeRepositoryImpl) Update(user *UserLevelTaskType) error {
	_, err := repo.o.Update(user)
	return err
}

// normally update user.deleted = 1 to soft-delete the record.
func (repo *userLevelTaskTypeRepositoryImpl) Delete(user *UserLevelTaskType) error {
	_, err := repo.o.Delete(user)
	return err
}
